from datetime import datetime, timedelta

import numpy as np
import pandas as pd
from ovretl.tracking_utils.check_shipment_active import check_shipment_active

empty_events_df = pd.DataFrame(data={"shipment_id": []})


def test_check_shipment_active():
    quotation = pd.Series(data=["0", "to_be_requoted", False], index=["shipment_id", "shipment_status", "cancelled"])
    assert not check_shipment_active(quotation, empty_events_df)
    finished = pd.Series(data=["0", "finished", False], index=["shipment_id", "shipment_status", "cancelled"])
    assert not check_shipment_active(finished, empty_events_df)
    arrival_date_active = pd.Series(
        data=["0", "in_progress", np.nan, np.nan, np.nan, datetime.now() + timedelta(days=5), np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(arrival_date_active, empty_events_df)
    arrival_date_not_active = pd.Series(
        data=["0", "booking_request", np.nan, np.nan, np.nan, datetime.now() + timedelta(days=11), np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert not check_shipment_active(arrival_date_not_active, empty_events_df)
    departure_date_active = pd.Series(
        data=["0", "in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(departure_date_active, empty_events_df)
    departure_date_not_active = pd.Series(
        data=["0", "in_progress", np.nan, np.nan, datetime.now() - timedelta(days=5), np.nan, np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert not check_shipment_active(departure_date_not_active, empty_events_df)
    is_arrived = pd.Series(
        data=["0", "in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert not check_shipment_active(
        is_arrived, pd.DataFrame(data={"shipment_id": ["0", "0"], "date_type": ["actual", "actual"]})
    )
    is_not_arrived_and_active = pd.Series(
        data=["0", "in_progress", np.nan, np.nan, datetime.now() - timedelta(days=1), np.nan, np.nan, False],
        index=[
            "shipment_id",
            "shipment_status",
            "finished_date",
            "pickup_date",
            "departure_date",
            "arrival_date",
            "delivery_date",
            "cancelled",
        ],
    )
    assert check_shipment_active(
        is_not_arrived_and_active, pd.DataFrame(data={"shipment_id": ["0", "0"], "date_type": ["actual", "estimated"]})
    )



import pandas as pd
from ovretl.prices_utils.find_most_relevant_prices import find_prices_from_entity
from pandas.util.testing import assert_frame_equal


def test_find_most_relevant_prices():
    entity_df = pd.DataFrame(data={"foo_id": [2, 1, 3],})
    prices_df = pd.DataFrame(data={"bar_id": [2, 5, 2, 6, 3],})
    result = find_prices_from_entity(
        entity_df=entity_df, prices_df=prices_df, entity_key_in_entity_df="foo_id", entity_key_in_prices_df="bar_id",
    )
    result_should_be = pd.DataFrame(data={"bar_id": [2, 2, 3]})
    assert_frame_equal(result.reset_index(drop=True), result_should_be)



import pandas as pd
from ovretl.transit_times.location_functions import is_import, is_abroad


def add_features(train_df):
    train_df_copy = train_df.copy()
    train_df_copy.loc[:, "is_import"] = train_df_copy.apply(is_import, axis=1)
    pickup_df = train_df_copy[
        [
            "freight_method",
            "load_type",
            "pickup_harbor_country",
            "pickup_harbor_locode",
            "pickup_shipowner",
            "pickup_distance",
            "pickup_time",
            "departure_shipowner",
            "freight_shipowner",
            "is_import",
        ]
    ]
    pickup_df = pickup_df.rename(
        columns={
            "pickup_harbor_country": "harbor_country",
            "pickup_harbor_locode": "harbor_locode",
            "pickup_shipowner": "carriage_shipowner",
            "pickup_distance": "distance",
            "pickup_time": "time",
            "departure_shipowner": "harbor_shipowner",
        }
    ).dropna(subset=["time"])
    pickup_df.loc[:, "carriage_type"] = "precarriage"
    delivery_df = train_df_copy[
        [
            "freight_method",
            "load_type",
            "freight_shipowner",
            "arrival_shipowner",
            "delivery_harbor_country",
            "delivery_harbor_locode",
            "delivery_shipowner",
            "delivery_distance",
            "delivery_time",
            "is_import",
        ]
    ]
    delivery_df = delivery_df.rename(
        columns={
            "arrival_shipowner": "harbor_shipowner",
            "delivery_harbor_country": "harbor_country",
            "delivery_harbor_locode": "harbor_locode",
            "delivery_shipowner": "carriage_shipowner",
            "delivery_distance": "distance",
            "delivery_time": "time",
        }
    ).dropna(subset=["time"])
    delivery_df.loc[:, "carriage_type"] = "postcarriage"
    train_df_copy = pd.concat([pickup_df, delivery_df], sort=False)
    train_df_copy.loc[:, "is_abroad"] = train_df_copy.apply(
        lambda s: is_abroad(s["is_import"], s["carriage_type"]), axis=1
    )
    return train_df_copy



import numpy as np
import pandas as pd
from ovretl.tracking_utils.check_shipment_active import check_shipment_active


def extract_event_date_by_sorting(events_shipment_df: pd.DataFrame, ascending: bool, location_key=None):
    if len(events_shipment_df) == 0:
        return np.nan
    events_shipment = events_shipment_df.sort_values(by="date", ascending=ascending).reset_index(drop=True)
    if len(events_shipment) > 0 and location_key is not None:
        if events_shipment.loc[0, "location_type"] == location_key:
            return (
                pd.to_datetime(events_shipment.loc[0, "date"])
                if not pd.isna(events_shipment.loc[0, "date"])
                else np.nan
            )
        return np.nan
    if len(events_shipment) > 0:
        return pd.to_datetime(events_shipment.loc[0, "date"]) if not pd.isna(events_shipment.loc[0, "date"]) else np.nan
    return np.nan


def extract_pickup_event_date(events_shipment_df: pd.DataFrame):
    mask_event_pickup = events_shipment_df["event_description"].apply(lambda e: "Pickup" in e)
    mask_event_not_delivery = events_shipment_df["event_description"].apply(lambda e: "Delivery" not in e)
    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_pickup], ascending=True,
    )
    event_found_by_order = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_delivery], ascending=True, location_key="warehouse",
    )
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    return event_found_by_order


def extract_departure_event_date(events_shipment_df: pd.DataFrame):
    mask_etd = events_shipment_df["is_used_for_etd"] == True
    mask_event_departure = events_shipment_df["event_description"].apply(lambda e: "Departure" in e)
    mask_event_not_arrival = events_shipment_df["event_description"].apply(lambda e: "Arrival" not in e)
    event_found_by_etd = extract_event_date_by_sorting(events_shipment_df=events_shipment_df[mask_etd], ascending=True,)

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_departure], ascending=True,
    )

    event_first = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_arrival], ascending=True, location_key="harbor",
    )
    if not pd.isna(event_found_by_etd):
        return event_found_by_etd
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    return event_first


def extract_arrival_event_date(events_shipment_df: pd.DataFrame):
    mask_eta = events_shipment_df["is_used_for_eta"] == True
    mask_event_arrival = events_shipment_df["event_description"].apply(lambda e: "Arrival" in e)
    mask_event_not_departure = events_shipment_df["event_description"].apply(lambda e: "Departure" not in e)
    events_shipment = events_shipment_df[mask_eta].sort_values(by="date", ascending=False).reset_index(drop=True)
    if len(events_shipment) > 0:
        return events_shipment.loc[0, "date"]

    event_found_by_eta = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_eta], ascending=False,
    )

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_arrival], ascending=False,
    )

    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_departure], ascending=False, location_key="harbor",
    )
    if not pd.isna(event_found_by_eta):
        return event_found_by_eta
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    return event_last


def extract_delivery_event_date(events_shipment_df: pd.DataFrame):
    mask_event_delivery = events_shipment_df["event_description"].apply(lambda e: "Delivery" in e)
    mask_event_not_pickup = events_shipment_df["event_description"].apply(lambda e: "Pickup" not in e)

    event_found_by_description = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_delivery], ascending=False,
    )

    event_last = extract_event_date_by_sorting(
        events_shipment_df=events_shipment_df[mask_event_not_pickup], ascending=False, location_key="warehouse",
    )
    if not pd.isna(event_found_by_description):
        return event_found_by_description
    return event_last


def add_tracking_event_dates(shipments_df: pd.DataFrame, events_shipment_df: pd.DataFrame) -> pd.DataFrame:
    pickup_dates = events_shipment_df.groupby("shipment_id").apply(extract_pickup_event_date)
    departure_dates = events_shipment_df.groupby("shipment_id").apply(extract_departure_event_date)
    arrival_dates = events_shipment_df.groupby("shipment_id").apply(extract_arrival_event_date)
    delivery_dates = events_shipment_df.groupby("shipment_id").apply(extract_delivery_event_date)

    shipments_with_events_df = pd.merge(
        shipments_df, pickup_dates.to_frame("pickup_date"), left_on="shipment_id", right_index=True, how="left",
    )

    shipments_with_events_df = pd.merge(
        shipments_with_events_df,
        departure_dates.to_frame("departure_date"),
        left_on="shipment_id",
        right_index=True,
        how="left",
    )

    shipments_with_events_df = pd.merge(
        shipments_with_events_df,
        arrival_dates.to_frame("arrival_date"),
        left_on="shipment_id",
        right_index=True,
        how="left",
    )

    shipments_with_events_df = pd.merge(
        shipments_with_events_df,
        delivery_dates.to_frame("delivery_date"),
        left_on="shipment_id",
        right_index=True,
        how="left",
    )
    shipments_with_events_df.loc[:, "is_active"] = shipments_with_events_df.apply(
        lambda s: check_shipment_active(s, events_shipment_df), axis=1
    )
    return shipments_with_events_df



import pandas as pd


def sum_category_price(df_prices: pd.DataFrame, category=None, key="price_in_eur"):
    if df_prices is None:
        return 0
    if category is not None:
        mask_category = df_prices["category"] == category
        df_prices = df_prices[mask_category]
        return df_prices[mask_category][key].sum()
    return df_prices[key].sum()


def select_category_price(df_prices, category, key):
    return df_prices.loc[category, key] if category in df_prices.index else 0



import pandas as pd


def pivot_category_prices(prices_df):
    prices_df = pd.pivot_table(prices_df, index=["shipment_id"], columns=["category"], values=["price_in_eur"])
    prices_df.columns = prices_df.columns.droplevel()
    return prices_df.rename(
        columns={
            "arrival_fees": "arrival_fees_price",
            "arrival_truck_freight": "arrival_truck_freight_price",
            "customs": "customs_price",
            "departure_fees": "departure_fees_price",
            "departure_truck_freight": "departure_truck_freight_price",
            "freight": "freight_price",
            "insurance": "insurance_price",
            "other": "other_price",
        }
    )


def pivot_category_purchase_prices(prices_df):
    prices_df = pd.pivot_table(prices_df, index=["shipment_id"], columns=["category"], values=["purchase_price_in_eur"])
    prices_df.columns = prices_df.columns.droplevel()
    return prices_df.rename(
        columns={
            "arrival_fees": "arrival_fees_purchase_price",
            "arrival_truck_freight": "arrival_truck_freight_purchase_price",
            "customs": "customs_purchase_price",
            "departure_fees": "departure_fees_purchase_price",
            "departure_truck_freight": "departure_truck_freight_purchase_price",
            "freight": "freight_purchase_price",
            "insurance": "insurance_purchase_price",
            "other": "other_purchase_price",
        }
    )



from ovretl.billings_utils.add_billing_info import *
from ovretl.computed_quotations_utils.add_missing_categories import *
from ovretl.computed_quotations_utils.add_shipments_to_cq import *
from ovretl.containers_utils.calculate_entity_containers import *
from ovretl.containers_utils.calculate_single_container_teus import *
from ovretl.containers_utils.merge_shipments_with_containers import *
from ovretl.employees_utils.find_shipment_employee_name import *
from ovretl.employees_utils.join_employees_associations_to_employees_name import *
from ovretl.loads_utils.calculate_entity_loads import *
from ovretl.loads_utils.calculate_single_load_total_quantities import *
from ovretl.loads_utils.merge_shipments_with_loads import *
from ovretl.prices_utils.add_category_prices_to_shipments import *
from ovretl.prices_utils.add_prices_to_shipments import *
from ovretl.shipments_utils.match_shipments_propositions import *
from ovretl.shipowners_utils.find_shipment_category_shipowner import *
from ovretl.tracking_utils.extract_event_date import *
from ovretl.tracking_utils.join_tracking_events import *
from ovretl.transit_times.preprocess_transit_times import *



import pandas as pd
import numpy as np
from typing import List

from ovretl.transit_times.constants import DEFAULT_ENCODING


def encode_distance(train_df: pd.DataFrame, group_size_threshold: int, feature: str):
    train_df.loc[:, feature] = train_df[feature].apply(lambda x: int(x / 100) * 100 if not pd.isna(x) else np.nan)
    value_counts = train_df[feature].value_counts()
    mapping = {key: key if value >= group_size_threshold else DEFAULT_ENCODING for key, value in value_counts.items()}
    train_df.loc[:, feature] = train_df[feature].replace(mapping)


def encode_feature(train_df: pd.DataFrame, group_size_threshold: int, feature: str):
    value_counts = train_df[feature].value_counts()
    mapping = {key: key if value >= group_size_threshold else DEFAULT_ENCODING for key, value in value_counts.items()}
    train_df.loc[:, feature] = train_df[feature].replace(mapping)


def encode_features(train_df: pd.DataFrame, group_size_threshold: int):
    qualitative_features = [
        "pickup_harbor_country",
        "pickup_harbor_locode",
        "delivery_harbor_country",
        "delivery_harbor_locode",
        "pickup_shipowner",
        "departure_shipowner",
        "arrival_shipowner",
        "delivery_shipowner",
        "freight_shipowner",
    ]
    distance_features = ["pickup_distance", "delivery_distance"]
    train_df_copy = train_df.copy()
    for feature in qualitative_features:
        encode_feature(train_df_copy, group_size_threshold, feature)
    for feature in distance_features:
        encode_distance(train_df_copy, group_size_threshold, feature)
    return train_df_copy
